//---------------------------------------------------------------------------------------------
/// \file	Main.cpp
/// \date	12-Jan-2005
/// \author	Rob Bateman
/// \brief	This plugin example demonstrates the usage of MModelMessage in watching for selection
/// 		changes of node duplications.
//---------------------------------------------------------------------------------------------

// required to link the libraries
#ifdef WIN32
	#define NT_PLUGIN
	#pragma comment(lib,"Foundation.lib")
	#pragma comment(lib,"OpenMaya.lib")
	#pragma comment(lib,"OpenMayaFx.lib")
	#pragma comment(lib,"OpenMayaUi.lib")
	#pragma comment(lib,"Image.lib")
	#pragma comment(lib,"OpenMayaAnim.lib")
#endif

// under WIN32 we have to 'export' these functions so that they are visible
// inside the dll. If maya can't see the functions, it can't load the plugin!
#ifdef WIN32
	#define EXPORT __declspec(dllexport)
#else
	#define EXPORT
#endif

#include <maya/MFnPlugin.h>
#include <maya/MModelMessage.h>
#include <maya/MGlobal.h>
#include <maya/MString.h>


//---------------------------------------------------------------------------------------------
///	\brief	This class creates a simple object that has a callback for whenever Maya's selection
/// 		list changes.
///
class SelectionWatcher {
public:

	/// \brief	ctor	-	creates the model callback
	///
	SelectionWatcher() ;

	/// dtor	-	unregisters the callback functions
	///
	~SelectionWatcher() ;

protected:

	/// \brief	This function can be used to update any instance of this class type.
	/// 		Essentially, the static function, Changed(), recieves a pointer
	/// 		to this instance, and calls this OnChanged member function.
	///
	virtual void OnChanged() {
		MGlobal::displayInfo("Selection List Changed");
	}

private:


	/// the callback ID
	MCallbackId m_Callback;

	/// \brief	This function calls
	static void Changed(void* data);
};



//---------------------------------------------------------------------------------------------
//
void SelectionWatcher::Changed(void* data) {

	( (SelectionWatcher*)data )->OnChanged();
}

//---------------------------------------------------------------------------------------------
//
SelectionWatcher::SelectionWatcher() {
	m_Callback = MModelMessage::addCallback( MModelMessage::kActiveListModified, Changed, this );
}

//---------------------------------------------------------------------------------------------
//
SelectionWatcher::~SelectionWatcher() {
	MModelMessage::removeCallback( m_Callback );
}




//---------------------------------------------------------------------------------------------
///	\brief	This class creates a simple object that has a callback for whenever Maya duplicates
/// 		any objects in the scene
///
class DuplicateWatcher {
public:

	/// \brief	ctor	-	creates the model callbacks
	///
	DuplicateWatcher() ;

	/// dtor	-	unregisters the callback functions
	///
	~DuplicateWatcher() ;

protected:

	/// \brief	This function is called immediately before objects in the scene are duplicated
	///
	virtual void PreDuplicate() {
		MGlobal::displayInfo("PreDuplicate");
	}

	/// \brief	This function is called immediately after objects in the scene are duplicated
	///
	virtual void PostDuplicate() {
		MGlobal::displayInfo("PostDuplicate");
	}

private:


	/// the callback ID for pre-dupliucate
	MCallbackId m_CB_PreDuplicate;

	/// the callback ID for post-dupliucate
	MCallbackId m_CB_PostDuplicate;

	// the static callback functions
	static void SPreDuplicate(void* data);
	static void SPostDuplicate(void* data);
};


//---------------------------------------------------------------------------------------------
//
void DuplicateWatcher::SPreDuplicate(void* data) {

	( (DuplicateWatcher*)data )->PreDuplicate();
}
void DuplicateWatcher::SPostDuplicate(void* data) {

	( (DuplicateWatcher*)data )->PostDuplicate();
}



//---------------------------------------------------------------------------------------------
//
DuplicateWatcher::DuplicateWatcher() {

	/// register the callback function with maya. for duplication, we use the following
	/// two custom functions
	///
	m_CB_PreDuplicate = MModelMessage::addBeforeDuplicateCallback( SPreDuplicate, this );
	m_CB_PostDuplicate = MModelMessage::addAfterDuplicateCallback( SPostDuplicate, this );
}

//---------------------------------------------------------------------------------------------
//
DuplicateWatcher::~DuplicateWatcher() {
	/// make sure we delete the callback before the plugin quits !!!
	/// Maya is likely to crash otherwise!
	MModelMessage::removeCallback( m_CB_PreDuplicate );
	MModelMessage::removeCallback( m_CB_PostDuplicate );
}


/// pointers to our objects with callbacks
SelectionWatcher* selectionMonitor;
DuplicateWatcher* duplicateMonitor;



/// \brief	This function is called once when our plugin is loaded. We can use
///			it to register any custom nodes, mel functions etc.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus initializePlugin( MObject obj )
{
	// we need to use the plugin function set to register a new mel command with maya.
	// we can provide some info about the author of the plugin, the version number, and we
	// can also specify a required version of maya, though usually "Any" will do...
	//
	MFnPlugin fnPlugin( obj, "Rob Bateman", "1.0", "Any");

	// create a monitor for the selection, and create a monitor for the duplication
	selectionMonitor = new SelectionWatcher;
	duplicateMonitor = new DuplicateWatcher;

	return MS::kSuccess;
}

/// \brief	This function is called once when our plugin is unloaded. We need
/// 		to tell maya which mel funcs, nodes etc we are removing.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus uninitializePlugin( MObject obj)
{
	// use the plugin function set to unload the plugin
	MFnPlugin fnPlugin( obj );

	// delete the selection & duplicate watchers
	delete selectionMonitor;
	delete duplicateMonitor;

	return MS::kSuccess;
}


